'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var to = require('to-case');
var objectToGcode = require('./objectToGcode');

var GcodeObject = function () {
  function GcodeObject() {
    _classCallCheck(this, GcodeObject);

    this.command = undefined;
    this.args = {};
    this.comment = undefined;
    this.commentTag = {
      command: undefined,
      args: {}
    };
  }

  _createClass(GcodeObject, [{
    key: 'toGcode',
    value: function toGcode(args) {
      return objectToGcode(this, args);
    }
  }]);

  return GcodeObject;
}();

/*
 * Type of command is held within <<< >>>
 * Afterwards args are passed
 * The arguments are either a value, or a key value pair
 * If the args are a single value, then a key value pair is made where the command is the key
 */


function parseCommentTag(comment) {
  var commentTag = {
    command: undefined,
    args: {}
  };

  var commentTagRegex = /.*<<<(.+)\s*>>>\s*(.*)/;
  var result = comment.match(commentTagRegex);

  // Don't parse the result unless you have data
  if (result && result.length >= 2 && result[1]) {
    var metaCommand = to.camel(result[1].toLowerCase());
    commentTag.command = metaCommand;

    if (result[2]) {
      var commentTagArgs = result[2].toLowerCase();
      if (commentTagArgs.includes(':')) {
        var commentTagKeyValue = commentTagArgs.split(':');
        var key = to.camel(commentTagKeyValue[0]);

        // Parse to number, if it's a number
        var value = Number.isNaN(Number(commentTagKeyValue[1])) ? commentTagKeyValue[1] : Number(commentTagKeyValue[1]);

        commentTag.args = JSON.parse('{"' + key + '": ' + value + '}');
      } else {
        commentTag.args = JSON.parse('{"' + metaCommand + '": ' + commentTagArgs + '}');
      }
    } else {
      commentTag.args = JSON.parse('{"' + metaCommand + '": true}');
    }
  }

  return commentTag;
}

/*
 * Parses a line of GCode and returns an object
 * Expects a single line of gcode
 * Returns an object with a command and a list of arguments
 */
function gcodeToObject(gcode) {
  // Valdate input to be of type "string"
  if (typeof gcode !== 'string') {
    throw new Error('gcode argument must be of type "string". ' + gcode + ' is type "' + (typeof gcode === 'undefined' ? 'undefined' : _typeof(gcode)) + '"');
  }

  // Constructing a blank gcode object, with functions
  var gcodeObject = new GcodeObject();

  // Split the gcode by the first semicolon it sees
  var commentSplits = gcode.split(';');
  var gcodeWithoutComment = commentSplits[0];
  if (commentSplits.length > 1) {
    var comment = commentSplits.slice(1).join(';');
    gcodeObject.comment = comment;
    gcodeObject.commentTag = parseCommentTag(comment);
  }

  // If we can find a command, assign it, otherwise keep the "command" value set to undefined
  var commandRegex = /[GM]\d+/;
  var commandResult = gcodeWithoutComment.toUpperCase().match(commandRegex);
  gcodeObject.command = (commandResult !== undefined || commandResult !== null) && Array.isArray(commandResult) && commandResult.length > 0 ? commandResult[0] : undefined;

  // Set the gcode to lower case and remove any G<number> or M<number> commands
  var gcodeArgString = gcodeWithoutComment.toLowerCase().replace(/[gm]\d+/, '');

  // Parse each axis for a trailing floating number
  // If no float, treat the axis as a boolean flag
  var axes = 'abcdefghijklmnopqrstuvwxyz'.split('');
  axes.forEach(function (axis) {
    // In most cases we are looking for an axis followed by a number
    var axisAndFloatRegex = new RegExp(axis + '\\s*([+-]?([0-9]*[.])?[0-9]+)');
    var result = gcodeArgString.match(axisAndFloatRegex);
    if (result) {
      gcodeObject.args[axis] = Number(result[1]);
      // If there is an axis, but no trailing number, pass the axis as a boolean flag
    } else if (gcodeArgString.includes(axis)) {
      gcodeObject.args[axis] = true;
    }
  });

  return gcodeObject;
}

module.exports = gcodeToObject;