'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _ = require('lodash');

/*
 * Parses a JSON object and returns a line of GCode
 * Expects a JSON object with each key value pair representing an axis and its corresponding value
 * Returns a Gcode string, without a line break
 */
function objectToGcode(gcodeObject) {
  var args = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  args.comment = args.comment == undefined ? true : args.comment;
  args.precision = args.precision == undefined ? 16 : args.precision;

  // Looking for 16 significant digits, aka only removing floating point madness
  var epsilon = args.precision;

  // Validate input to be of type "object"
  if ((typeof gcodeObject === 'undefined' ? 'undefined' : _typeof(gcodeObject)) !== 'object') {
    throw new Error('Input argument must be of type "object". ' + gcodeObject + ' is type "' + (typeof gcodeObject === 'undefined' ? 'undefined' : _typeof(gcodeObject)) + '"');
  }

  // Create the gcode string.
  // Start with the command
  // Followed by each axis
  var gcode = '';
  if (gcodeObject.command) {
    gcode += gcodeObject.command;
  }

  // Process extrusion and then feedrate at the end of the commands, if at all
  var processE = false;
  var processF = false;

  _.entries(gcodeObject.args).forEach(function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        key = _ref2[0],
        value = _ref2[1];

    if (key === 'e') {
      processE = true;
    } else if (key === 'f') {
      processF = true;
    } else {
      if (gcode.length > 0) {
        gcode += ' ';
      }
      gcode += key.toUpperCase();
      if (typeof value !== 'boolean') {
        // TODO, allow user to pass prefered precision for any axis or all axes
        var roundedNumber = Number(value.toFixed(epsilon));
        gcode += roundedNumber;
      }
    }
  });

  if (processE) {
    if (gcode.length > 0) {
      gcode += ' ';
    }
    var roundedNumber = Number(gcodeObject.args.e.toFixed(epsilon));
    gcode += 'E' + roundedNumber;
  }

  if (processF) {
    if (gcode.length > 0) {
      gcode += ' ';
    }
    var _roundedNumber = Number(gcodeObject.args.f.toFixed(epsilon));
    gcode += 'F' + _roundedNumber;
  }

  if (gcodeObject.comment && args.comment !== false) {
    gcode += ';';
    gcode += gcodeObject.comment;
  }

  return gcode;
}

module.exports = objectToGcode;